function [PHI, muPHI, CovPHI, s2] = basis_stats(Data, Coord, Var, Per)

iwih = size(Data, 2);  %image vector size (image width x image height)

nv = max(Var);         %maximum number of variation type in the bootstrap database
nsb = size(Coord, 2);  %the size of subspace basis (number of vectors in the basis)
np = max(Per);         %number of people used in training

tic
fprintf( '\tCalculating subspace bases                 ');
%Calculate the covariance matrix emprically
A = Coord'*Coord;
PHI = zeros(iwih, (nsb)*np);
for i = 1 : np
    b = Coord(Var(Per==i), :)' * Data(Per==i, :);
    PHI(:, (i-1)*(nsb)+1:i*(nsb)) = (A\b)';
end

CovPHI = zeros(iwih, (nsb)^2);
div = zeros(nv, iwih);
for i = 1 : iwih
    TT = PHI(i, :);
    TT = reshape(TT', nsb, np)';
    TT = cov(TT);
    TT = TT + eye(size(TT))*1e-7;
    CovPHI(i, :) = TT(:)';
    div(:, i) = diag(Coord*TT*Coord');
end

C = zeros(size(Data, 1), nsb);
for i = 1 : np
    C(Per==i, :) = Coord(Var(Per==i), :);
end
X = zeros(nv, iwih);
for i = 1 : nv
    X(i, :) = mean(Data(Var==i, :));
end

%Main loop to calculate mean for basis vector and variance of the error
%term
s = ones(nv, iwih);
sn = zeros(nv, iwih);
muPHI = zeros(iwih, nsb);
for it = 1 : 100
    for n = 1 : iwih
        A = Coord'*( Coord ./  repmat(s(:, n), 1, nsb));
        b = Coord' * (X(:, n) ./ s(:, n));
        muPHI(n, :) = A\b;
    end
    
    A = (Data - C*muPHI').^2;
    for n = 1 : nv
        sn(n, :) = mean(A(Var==n, :));
    end
    
    sn(sn<div) = div(sn<div) + 1e-6;
    
    err = norm(s-sn);
    s = sn;
    if err < 1e-6
        break
    end
end

s2 = (s - div)';

fprintf( 'DONE with %d iterations and error of %0.6f (%0.2f)\n', it, err, toc);