function [Coord, mCoord, Mapping] = lpp(Data, Label, nv, nsb)

% You may check the details on this part in ../sidekick/LPP.m file

%Reference:
%	Xiaofei He, and Partha Niyogi, "Locality Preserving Projections"
%	Advances in Neural Information Processing Systems 16 (NIPS 2003),
%	Vancouver, Canada, 2003.
%
%   Xiaofei He, Shuicheng Yan, Yuxiao Hu, Partha Niyogi, and Hong-Jiang
%   Zhang, "Face Recognition Using Laplacianfaces", IEEE PAMI, Vol. 27, No.
%   3, Mar. 2005. 
%
%   Deng Cai, Xiaofei He and Jiawei Han, "Document Clustering Using
%   Locality Preserving Indexing" IEEE TKDE, Dec. 2005.
%
%   Deng Cai, Xiaofei He and Jiawei Han, "Using Graph Model for Face Analysis",
%   Technical Report, UIUCDCS-R-2005-2636, UIUC, Sept. 2005
% 
%	Xiaofei He, "Locality Preserving Projections"
%	PhD's thesis, Computer Science Department, The University of Chicago,
%	2005.


%PART-1 Apply LPP Method
%==========================================================================
options = [];
options.PCARatio = 0.9;
options.Metric = 'Euclidean';
options.NeighborMode = 'Supervised';
options.k = 3;
options.WeightMode = 'HeatKernel';
options.t = 1e+6;
options.bSelfConnected = 0;
options.gnd = Label';

tic
fprintf( '\tCalculating distances                      ');
W = constructW(Data,options);
fprintf( 'DONE (%0.2f)\n', toc);

tic
fprintf( '\tCalculating reduced coordinates            ');
[eigvector, eigvalue] = LPP(W, options, Data);

nsb = min(size(eigvector, 2), nsb);

Mapping = eigvector(:,1:nsb);
Coord = Data*Mapping;
fprintf( 'DONE (%0.2f)\n', toc);

%PART-2 Find Center Coordinates For Each Illumination Code
%==========================================================================
mCoord = zeros(nv, nsb);
for i = 1 : nv
    mCoord(i, :) = mean(Coord(Label==i, :));
end
